package devmgr.api.rpc;

import java.io.IOException;
import java.net.InetAddress;
import java.util.Date;

import devmgr.api.rpc.protocol.RPCAuthType;
import devmgr.api.rpc.protocol.RPCCallMsg;
import devmgr.api.rpc.protocol.RPCMsg;
import devmgr.api.rpc.protocol.RPCMsgBody;
import devmgr.api.rpc.protocol.RPCMsgType;
import devmgr.api.rpc.protocol.RPCOpaqueAuth;
import devmgr.api.rpc.protocol.RPCProtocol;
import devmgr.api.rpc.protocol.RPCReplyStatus;
import devmgr.api.runtime.ApiDefinition;
import devmgr.api.runtime.ApiDefinition.ProcTableEntry;
import devmgr.api.runtime.Decoder.TypeOverrideTable;
import devmgr.api.runtime.DecodeException;
import devmgr.api.runtime.Decoder;
import devmgr.api.runtime.EncodeException;
import devmgr.api.runtime.TypeSpec;
import devmgr.api.runtime.XdrDecoder;
import devmgr.api.runtime.XdrEncoder;
import devmgr.api.runtime.XdrMessage;

/**
 * This class implements a generic RPC client, without regard to
 * the type of protocol being used.  It must be subclassed by a
 * true protocol provider to be useful.  All of the core methods for
 * building RPC request messages, handling replies, etc., are included
 * as part of this class.
 */
public abstract class RPCClientGeneric
{
    private ApiDefinition m_ApiDef;

    private XdrProtocolHelper m_protocolHelper;


    /**
     * We use a timestamp-based transaction ID value that gets incremented
     * with each server interaction.  The initial value is set using the
     * system clock at the time our constructor is called.
     */
    private int m_NextXid;


    /**
     * Constructor for generic RPC client objects.  The RPC program
     * number and version number are specified as arguments.
     * @param apiDef API definition for the program.
     */
    public RPCClientGeneric(ApiDefinition apiDef)
    {
        m_ApiDef = apiDef;
        m_protocolHelper=new XdrProtocolHelper(apiDef);

        // Set up the transaction ID sequencer using the current time
        // of day value from the system clock.
        m_NextXid = (int) ( (new Date()).getTime() / 1000 );

        return;
    }

    /**
     * Perform a call/return sequence on the specified remote procedure,
     * using the argument and result variables given by the user.  Handle
     * all logic associated with building the appropriate request message
     * and decoding the reply.
     * @param proc Procedure being called.
     * @param arg Argument being passed to the procedure.
     *    this data is not needed.
     * @exception RPCError If any RPC error occurs.
     * @exception IOException If any underlying network error occurs.
     */
    public synchronized Object call(int proc, Object arg) 
    throws RPCError, IOException, EncodeException, DecodeException
    {
        int xid = getNextXid();

        XdrMessage callMsg = m_protocolHelper.buildRequestMsg(proc, xid, arg);
        XdrMessage replyMsg = transact(callMsg, xid);
        return m_protocolHelper.processReplyMsg(proc, replyMsg, xid);
    }
    
    /**
     * Sets the type-override table that is used when decoding reply
     * messages from the remote server.
     * @param table Type override table to use.
     */
    public synchronized void setTypeOverrides(TypeOverrideTable table)
    {
        m_protocolHelper.setTypeOverrides(table);
    }

    /**
     * Sets the authentication-generator object that is used by the client
     * when constructing outbound request messages.  This allows the user
     * of the object to plug in a customized generator of authentication
     * information.
     * @param generator New authentication generator object to be used.
     */
    synchronized void setAuthGenerator(RPCAuthGenerator generator)
    {
       m_protocolHelper.setAuthGenerator(generator);
    }

    /**
     * Close the underlying network connection.
     * @exception IOException If any underlying network error occurs.
     */
    public abstract void close()
    throws IOException;

    /**
     * Sets the timeout limit for the underlying connection.
     * @param seconds Number of seconds to wait before timing out.
     * @exception IOException If an I/O error occurs on the connection.
     */
    public abstract void setTimeout(int seconds)
    throws IOException;

    /**
     * Returns the IP address of the client end of the RPC connection.
     * This is particularly useful for clients running on multi-homed
     * hosts, where it's not obvious what local IP address is being
     * used by the client-side network socket.
     * @return Client-side IP address.
     */
    public abstract InetAddress getLocalAddress();

    /**
     * Returns the IP address of the server end of the RPC connection.
     * @return Server-side IP address.
     */
    public abstract InetAddress getDestinationIP();

    /**
     * Performs an RPC call/reply transaction for this connection.
     * Sends the specified call message, then waits for (and returns
     * to the caller with) the corresponding reply message.  This is an
     * abstract method that must be supplied by a derived class.
     * @param callMsg The outbound call message.
     * @param xid The transaction ID for this exchange.
     * @return The IOV-based message containing reply data from the server.
     * @exception RPCError If any RPC error occurs.
     * @exception IOException If an I/O error occurs on the connection.
     */
    public abstract XdrMessage transact(XdrMessage callMsg, int xid)
    throws RPCError, IOException;

    /**
     * Obtain the next valid transaction ID value for use with this RPC
     * client object.
     * @return The next transaction ID value to be used by this client.
     */
    protected synchronized int getNextXid()
    {
        return m_NextXid++;
    }




}
