package devmgr.api.runtime;

import devmgr.api.symbolutils.ByteUtils;

import java.util.LinkedList;

/**
 * This class implements an XDR encoder.  As part of its processing, all 
 * data elements are translated to network-encoded form from the local 
 * representation.  Internally, all data is accumulated in an XDRMessageIOV 
 * object. When the encoding is complete, that object can be obtained by
 * a client of this class and used for further operations, such as sending
 * data over a network.
 */
public class XdrEncoder
extends Encoder
{
    private static final int XDR_UNIT_SIZE = 4;
    private static final int XDR_BUFFER_SIZE = 1024;

    private final XdrMessage m_Msg;
    private byte[] m_Buf;
    private int m_OffsetInBuffer = XDR_BUFFER_SIZE;
    private int m_OffsetInMsg = 0;
    private int m_TotalMsgSize = 0;
    private int m_NextBufIndex = 0;
    private int m_TotalBufCount = 0;
    
    private final LinkedList<EncodeContext> m_ContextList =
        new LinkedList<EncodeContext>();
    
    private int m_NestingDepth = 0;
    private boolean m_Verbose = false;

    private static class EncodeContext
    {
        int prefixPos;
        int startPos;
    }

    /**
     * Constructor for XDREncoder objects.  Such objects are
     * typically used to gather encoded data for later transmission
     * on a network connection.  An internal message IOV is maintained,
     * and automatically enlarged when necessary, to store the encoded
     * data.  When the encoding process is complete, the user can
     * access the complete message by calling the getMessage() method.
     */
    public XdrEncoder()
    {
        m_Msg = new XdrMessage();
        
        m_Buf = null;
        m_NextBufIndex = 0;
        m_TotalBufCount = 0;
        m_OffsetInBuffer = XDR_BUFFER_SIZE;
        m_OffsetInMsg = 0;
        m_TotalMsgSize = 0;
        
        super.setEncodeNulls(true);
        super.setEncodeHiddenFields(true);

        return;
    }

    /**
     * Flush the underlying output stream.  This is a no-op, since
     * we don't use standard stream objects for our output.
     */
    @Override
    protected void
    flush()
    throws EncodeException
    {
    }
    
    /**
     * Sets the verbose/quiet mode.  When in verbose mode, state/debug
     * information is written to System.err.
     * @param verbose True for verbose output; false for quiet mode.
     */
    public void
    setVerbose(boolean verbose)
    {
        m_Verbose = verbose;
    }
    
    /**
     * Show state/debug information.
     * @param fmt Format string to use.
     * @param args Format arguments.
     */
    private void 
    showState(String fmt, Object... args)
    {
        if (! m_Verbose) {
            return;
        }
        
        System.err.print("XDR-E:");
        for (int i = 0; i < m_NestingDepth; i++) {
            System.err.print("  ");
        }
        
        System.err.printf(fmt, args);
        
        System.err.printf(" [pos=%d]%n", m_OffsetInMsg);
    }
    
    /**
     * Returns a reference to the message IOV so the caller can
     * access the encoded message contents.
     * @return The IOV-style representation of the constructed message.
     */
    public XdrMessage getMessage()
    {
        m_Msg.setMessageSize(m_TotalMsgSize);
        return m_Msg;
    }

    /**
     * Add the specified byte value to the stream.
     * @param value Byte value to be added
     */
    private void putByte(int value)
    {
        // If the buffer is full, handle it here.
        if (m_OffsetInBuffer >= XDR_BUFFER_SIZE)
        {
            // If we're overlaying previously-built buffers, move to
            // the next available one.
            if (m_NextBufIndex < m_TotalBufCount)
            {
                m_Buf = m_Msg.getBuffer(m_NextBufIndex++);
            }
            
            // Otherwise, we have to create a new buffer and append
            // it to the message.
            else
            {
                m_Buf = new byte[XDR_BUFFER_SIZE];
                m_Msg.addBuffer(m_Buf);
                m_NextBufIndex++;
                m_TotalBufCount++;
            }
            
            m_OffsetInBuffer = 0;
        }
            
        // Put the byte into the buffer now.
        m_Buf[m_OffsetInBuffer++] = (byte) value;
        m_OffsetInMsg++;
        if (m_OffsetInMsg > m_TotalMsgSize) {
            m_TotalMsgSize = m_OffsetInMsg;
        }

        return;
    }

    /**
     * Encodes a (32-bit) integer.
     * @param value The integer value to be encoded.
     */
    private void putInt(int value)
    {
        putByte(value >>> 24);
        putByte(value >>> 16);
        putByte(value >>>  8);
        putByte(value >>>  0);
    }

    /**
     * Encodes a (64-bit) long value.
     * @param value The long value to be encoded.
     */
    private void putLong(long value)
    {
        putByte((int) (value >>> 56));
        putByte((int) (value >>> 48));
        putByte((int) (value >>> 40));
        putByte((int) (value >>> 32));
        putByte((int) (value >>> 24));
        putByte((int) (value >>> 16));
        putByte((int) (value >>>  8));
        putByte((int) (value >>>  0));
    }

    /**
     * Encodes an opaque byte array.
     * @param buf The byte array to be added to the stream
     * @exception EncodeException If an underlying error occurs.
     */
    private void putOpaque(byte[] buf)
    throws EncodeException
    {
        int amt = buf.length;

        for (int i = 0; i < amt; i++) {
            putByte((int) buf[i]);
        }

        int pad = calculatePad(amt);
        for (int i = 0; i < pad; i++) {
            putByte(0);
        }

        return;
    }

    /**
     * Encodes an opaque byte array using a single-byte-per-char
     * string as the starting point.
     * @param s The string to encode
     * @param filler The number of filler bytes to use
     * @exception EncodeException If an underlying error occurs.
     */
    private void putSBCS(String s, int filler)
    throws EncodeException
    {
        for (int i = 0; i < s.length(); i++)
        {
            int c = (int) s.charAt(i);
            putByte((byte) c);
        }
        
        int pad = calculatePad(s.length() + filler);
        for (int p = 0; p < (filler + pad); p++) {
            putByte(0);
        }
        
        return;
    }

    /**
     * Encodes an opaque byte array using a double-byte-per-char
     * string as the starting point.
     * @param s The string to encode
     * @param filler The number of filler bytes to use
     * @exception EncodeException If an underlying error occurs.
     */
    private void putDBCS(String s, int filler)
    throws EncodeException
    {
        for (int i = 0; i < s.length(); i++)
        {
            int c = (int) s.charAt(i);
            putByte((byte) (c >>> 8));
            putByte((byte) c);
        }
        
        int pad = calculatePad((2 * s.length()) + filler);
        for (int p = 0; p < (filler + pad); p++) {
            putByte(0);
        }
        
        return;
    }

    /**
     * Determines the number of pad bytes associated with the
     * specified data length to obtain an even multiple of the
     * XDR_UNIT_SIZE value.
     * @param amt Amount of data used in operation
     * @return The amount of padding needed for the specified byte count.
     */
    private int calculatePad(int amt)
    {
        int residual = amt % XDR_UNIT_SIZE;

        if (residual == 0) {
            return 0;
        }
        else {
            return XDR_UNIT_SIZE - residual;
        }
    }

    /**
     * This method starts a new context, as needed when starting to
     * encode a new struct or union entity.
     * @throws EncodeException
     */
    private void startNewContext()
    throws EncodeException
    {
        EncodeContext encCtx = new EncodeContext();
        
        encCtx.prefixPos = m_OffsetInMsg;

        putInt(0);
        encCtx.startPos = m_OffsetInMsg;
        
        m_ContextList.add(encCtx);
    }

    /**
     * This method completes the current context by computing and 
     * patching-in the appropriate length value.
     * @throws EncodeException
     */
    private void finishContext()
    throws EncodeException
    {
        EncodeContext encCtx = m_ContextList.removeLast();

        int length = m_OffsetInMsg - encCtx.startPos;
        pokeInt(length, encCtx.prefixPos);
    }

    /**
     * This method stores an integer into the specified position in the
     * stream, which must have ALREADY been written in a previous encode
     * operation.
     * @param value Integer value to be stored.
     * @param pos Position at which to store the integer.
     */
    private void pokeInt(int value, int pos)
    {
        int bufNum = pos / XDR_BUFFER_SIZE;
        int offset = pos % XDR_BUFFER_SIZE;
        byte[] buf = m_Msg.getBuffer(bufNum);
        
        /*
         * Now encode the integer length value to the desired location.  We are
         * guaranteed that it will not span two buffers, since every encoded
         * object starts on an XDR_UNIT_SIZE boundary, and integers are 4-byte
         * entities (which match XDR_UNIT_SIZE).
         */
        buf[offset + 0] = (byte) (value >>> 24);
        buf[offset + 1] = (byte) (value >>> 16);
        buf[offset + 2] = (byte) (value >>> 8);
        buf[offset + 3] = (byte) (value >>> 0);
    
        return;
    }

    /**
     * Returns the current byte offset within the stream.
     * @return Current position (i.e. offset) in the stream.
     */
    public int getPosition()
    {
        return m_OffsetInMsg;
    }

    /**
     * Sets the position for further encoding operations to the specified
     * byte offset in the stream.  Note that this does not impact the
     * size of the message, as encoded thus far.
     * @param offset Desired position in stream for further encoding.
     */
    public void setPosition(int offset)
    {
        m_NextBufIndex = offset / XDR_BUFFER_SIZE;
        m_Buf = m_Msg.getBuffer(m_NextBufIndex++);

        m_OffsetInBuffer = offset % XDR_BUFFER_SIZE;
        
        m_OffsetInMsg = offset;
    }

    @Override
    protected void 
    encodeByte(byte obj, String name, Context ctx)
    throws EncodeException
    {
        putInt((int)(obj) & 0xFF);
    }

    @Override
    protected void 
    encodeShort(short obj, String name, Context ctx)
    throws EncodeException
    {
        putInt((int)(obj) & 0xFFFF);
    }

    @Override
    protected void 
    encodeInt(int obj, String name, Context ctx)
    throws EncodeException
    {
        putInt(obj);
    }

    @Override
    protected void 
    encodeLong(long obj, String name, Context ctx)
    throws EncodeException
    {
        putLong(obj);
    }

    @Override
    protected void 
    encodeBoolean(boolean obj, String name, Context ctx)
    throws EncodeException
    {
        putInt(obj ? 1 : 0);
    }

    @Override
    protected void 
    encodeFloat(float obj, String name, Context ctx)
    throws EncodeException
    {
        putInt(Float.floatToIntBits(obj));
    }

    @Override
    protected void 
    encodeDouble(double obj, String name, Context ctx)
    throws EncodeException
    {
        putLong(Double.doubleToLongBits(obj));
    }

    @Override
    protected void 
    encodeString(String obj, String name, Context ctx, int maxLength)
    throws EncodeException
    {
        if (obj == null)
        {
            putInt(0);
            return;
        }
        
        int length = obj.length();
        
        if (length > maxLength) {
            throw new EncodeException("XDR: string overflow");
        }
    
        putInt(length);
        
        for (int i = 0; i < length; i++) {
            putByte((int) obj.charAt(i));
        }

        int pad = calculatePad(length);
        for (int i = 0; i < pad; i++) {
            putByte(0);
        }
    }

    @Override
    protected void 
    encodeOpaque(String data, String name, Context ctx, OpaqueSpec typeSpec)
    throws EncodeException
    {
        if (data == null)
        {
            if (typeSpec.isVariable) {
                putInt(0);
            }
            else {
                putOpaque(new byte[typeSpec.maxLength]);
            }
            
            return;
        }
        
        int filler = 0;
        
        switch (typeSpec.dataType)
        {
            case BINARY:
                if ((data.length() % 2) != 0) {
                    throw new EncodeException("XDR: bad opaque data");
                }
                
                if (typeSpec.isVariable)
                {
                    if ((data.length() / 2) > typeSpec.maxLength) {
                        throw new EncodeException
                                ("XDR: variable opaque overflow");
                    }
                    
                    putInt(data.length() / 2);
                }
                
                else if ((data.length() / 2) != typeSpec.maxLength) {
                    throw new EncodeException("XDR: fixed opaque mismatch field:" + name);
                }
                    
                putOpaque(ApiUtil.stringToOpaque(data));
                break;
                
            case DBCS:
                if ((data.length() * 2) > typeSpec.maxLength) {
                    throw new EncodeException
                            ("XDR: variable opaque overflow");
                }

                if (typeSpec.isVariable) {
                    putInt(data.length() * 2);
                }
                else {
                    filler = typeSpec.maxLength - (data.length() * 2);
                }
                
                putDBCS(data, filler);
                break;
                
            case SBCS:
                if (data.length() > typeSpec.maxLength) {
                    throw new EncodeException
                            ("XDR: variable opaque overflow");
                }

                if (typeSpec.isVariable) {
                    putInt(data.length());
                }
                else {
                    filler = typeSpec.maxLength - data.length();
                }
                
                putSBCS(data, filler);
                break;
            case IPV4ADDR:
                byte[] bitData=ByteUtils.getIPByteArray(data);
                if (bitData == null) {
                    throw new EncodeException
                            ("XDR: variable opaque ipv4 invalid format");
                }
                for (byte b:bitData) {
                    putByte(b);
                }

        }
    }

    @Override
    protected void 
    encodeArrayStart(String name, Context ctx, 
                     boolean isVariable, int actualLength)
    throws EncodeException
    {
        if (isVariable) {
            putInt(actualLength);
        }
    }

    @Override
    protected void 
    encodeArrayEnd(String name, Context ctx, 
                   boolean isVariable, int actualLength)
    throws EncodeException
    {
    }

    @Override
    protected void 
    encodeStructStart(String name, Context ctx, StructSpec spec)
    throws EncodeException
    {
        showState("+struct: %s", name);
        m_NestingDepth++;
        
        if (spec.isVarLen) {
            startNewContext();
        }
    }

    @Override
    protected void 
    encodeStructEnd(String name, Context ctx, StructSpec spec)
    throws EncodeException
    {
        m_NestingDepth--;
        showState("-struct: %s", name);

        if (spec.isVarLen) {
            finishContext();
        }
    }

    @Override
    protected void 
    encodeUnionStart(String name, Context ctx, UnionSpec spec,
                     EnumValue discrimVal)
    throws EncodeException
    {
        showState("+union: %s", name);
        m_NestingDepth++;
        
        if (spec.isVarLen) {
            startNewContext();
        }

        encodeEnum(discrimVal, spec.discrimName, Context.NONE); 
    }

    @Override
    protected void 
    encodeUnionEnd(String name, Context ctx, UnionSpec spec)
    throws EncodeException
    {
        m_NestingDepth--;
        showState("-union: %s", name);
        
        if (spec.isVarLen) {
            finishContext();
        }
    }

    @Override
    protected void 
    encodeEnum(EnumValue enumVal, String name, Context ctx)
    throws EncodeException
    {
        putInt(enumVal.value);
    }

    @Override
    protected void 
    encodeEnumSet(EnumValue[] values, String name, Context ctx)
    throws EncodeException
    {
        int composite = 0;
        for (int i = 0; i < values.length; i++)
        {
            // Get the current value, which should be a bit mask.
            int mask = values[i].value;
            
            // If it's the special "bogus" value, just encode the
            // entire field using its value; i.e. throw away all
            // the other bits/masks.
            if (mask == ApiUtil.BOGUS_ENUM_VALUE)
            {
                composite = ApiUtil.BOGUS_ENUM_VALUE;
                break;
            }
            
            // Add this mask's bits to the composite value.
            composite |= mask;
        }
        
        putInt(composite);
    }

    @Override
    protected void 
    encodePointerPrefix(String name, Context ctx, boolean isNull)
    {
        if (isNull) {
            putInt(0);
        }
        else {
            putInt(1);
        }
    }
}