package devmgr.api.runtime;

import java.util.Vector;

/**
 * This class implements an object that represents an I/O-Vector
 * type of message buffer.  The message is actually held in one
 * or more buffers, which are considered logically concatenated.
 * Added buffers are always considered "full", but the caller can
 * adjust the logical length of the message such that it does not
 * span the entire set of buffers.
 */
public class XdrMessage
{
    private static final int INITIAL_BUFS = 32;
    private static final int INCREMENTAL_BUFS = 32;

    private boolean m_Capped = false;
    private Vector<byte[]> m_Buffers;
    private int m_MaxTotalSize;
    private int m_MessageSize;

    /**
     * Constructor for IOV-based message objects.  Creates a new,
     * zero-length message object that can later be populated with
     * concatenated buffers, and then queried for its contents.
     */
    public XdrMessage()
    {
        m_Buffers = new Vector<byte[]>(INITIAL_BUFS, INCREMENTAL_BUFS);
        m_MaxTotalSize = 0;
        m_MessageSize = 0;
    }

    /**
     * Get the size of the message represented by this object.
     * @return Number of bytes in the message as a whole.
     */
    public int getMessageSize()
    {
        return m_MessageSize;
    }

    /**
     * Set the size of the message represented by this object.  The
     * size value must be less than or equal to the sum of the sizes
     * of all added buffers.
     * @param size Desired new (logical) size of the message.
     */
    public void setMessageSize(int size)
    {
        m_MessageSize = Math.min(size,  m_MaxTotalSize);
    }

    /**
     * Get the specified buffer, which holds part of the message contents.
     * @param bufIndex Index of the desired buffer (0, 1, 2, etc.).
     * @return The specified buffer.
     */
    public byte[] getBuffer(int bufIndex)
    {
        return m_Buffers.elementAt(bufIndex);
    }

    /**
     * Copy all of the IOV data to the specified byte array, which must
     * be big enough to hold everything contiguously.
     * @arg dest Destination byte array.
     */
    public void copyData(byte[] dest)
    {
        int amtDone = 0;
        int nextBuf = 0;
        int amt;
        byte[] iovBuf;

        while (amtDone < m_MessageSize)
        {
            iovBuf = getBuffer(nextBuf++);
            amt = Math.min(iovBuf.length, m_MessageSize - amtDone);
            System.arraycopy(iovBuf, 0, dest, amtDone, amt);
            amtDone += amt;
        }
    }

    /**
     * Add a full buffer to the tail of an existing message.
     * @param buf Buffer to be added.
     */
    public void addBuffer(byte[] buf)
    {
        addBuffer(buf, buf.length);
    }

    /**
     * Add a potentially non-full buffer to the tail of an existing message. 
     * Once this is done, the message is capped, i.e. no more buffers can
     * be added to it.
     */
    public void addBuffer(byte[] buf, int amt)
    {
        if (m_Capped)
            throw new RuntimeException("XdrMessage: capped buffer add");
        
        if (amt != buf.length)
            m_Capped = true;
        
        m_Buffers.addElement(buf);
        m_MaxTotalSize += amt;
        m_MessageSize = m_MaxTotalSize;
    }
    
    /**
     * Get a buffer that contains all of the data in the message.
     * If possible, the returned buffer will just be a reference to the
     * first buffer in the IOV.  However, if multiple IOV elements are
     * being used to store the message, a new buffer will be allocated
     * to hold all of the data contiguously, and data will be copied
     * to it.
     * @return A single buffer that contains the entire message.
     */
    public byte[] getContiguousBuffer()
    {
        byte[] result;
        byte[] iovBuf;

        // If there's only one buffer in the IOV, use it.
        iovBuf = getBuffer(0);
        if (iovBuf.length >= m_MessageSize)
        {
            result = iovBuf;
        }

        // Otherwise, allocate a contiguous buffer and copy data into it.
        else
        {
            result = new byte[m_MessageSize];
            copyData(result);
        }

        return result;
    }
}
