package devmgr.api.runtime;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;

/**
 * This class implements XML encoding to an output writer.
 */
public class XmlEncoder
extends Encoder
{
    private Writer m_Out;
    
    /**
     * Constructor.
     * @param writer Writer to which results are sent.
     */
    public 
    XmlEncoder(Writer writer)
    {
        m_Out = writer;
    }

    /**
     * Constructor.
     * @param out Output stream to which results are sent.
     */
    public 
    XmlEncoder(OutputStream out)
    {
        this(new OutputStreamWriter(out, StandardCharsets.UTF_8));
    }
    
    /**
     * Flush the output stream.
     */
    @Override
    protected void
    flush()
    throws EncodeException
    {
        try
        {
            m_Out.flush();
        }
        catch (Exception ex)
        {
            throw EncodeException.build(ex, "Can't flush XML stream");
        }
    }
    
    /**
     * Getter method for the Writer object to which results are sent.
     * @return
     */
    public Writer
    getWriter()
    throws IOException
    {
        m_Out.flush();
        return m_Out;
    }
    
    /**
     * This method emits the specified string to the output stream.
     * @param s String to be emitted/written.
     * @throws EncodeException
     */
    private void 
    emitString(String s)
    throws EncodeException
    {
        try
        {
            m_Out.append(s);
        }
        catch (Exception ex)
        {
            throw EncodeException.build(ex, "Can't emit string");
        }
    }
    
    private void
    putOpenElement(String eltName)
    throws EncodeException
    {
        emitString(String.format("<%1$s>", eltName));
    }

    private void
    putCloseElement(String eltName)
    throws EncodeException
    {
        emitString(String.format("</%1$s>", eltName));
    }
    
    private void
    putElement(String eltName, String eltValue)
    throws EncodeException
    {
        emitString(String.format("<%1$s>%2$s</%1$s>", eltName, eltValue));
    }
    
    @Override
    protected void 
    encodeByte(byte obj, String name, Context ctx)
    throws EncodeException
    {
        encodeInt((int) obj, name, ctx);
    }
    
    @Override
    protected void 
    encodeShort(short obj, String name, Context ctx)
    throws EncodeException
    {
        encodeInt((int) obj, name, ctx);
    }

    @Override
    protected void 
    encodeInt(int obj, String name, Context ctx)
    throws EncodeException
    {
        putElement(name, Integer.toString(obj));
    }
    
    @Override
    protected void 
    encodeLong(long obj, String name, Context ctx)
    throws EncodeException
    {
        putElement(name, Long.toString(obj));
    }

    @Override
    protected void 
    encodeBoolean(boolean obj, String name, Context ctx)
    throws EncodeException
    {
        putElement(name, obj ? "true" : "false");
    }
    
    @Override
    protected void 
    encodeFloat(float obj, String name, Context ctx)
    throws EncodeException
    {
        putElement(name, Float.toString(obj));
    }
    
    @Override
    protected void 
    encodeDouble(double obj, String name, Context ctx)
    throws EncodeException
    {
        putElement(name, Double.toString(obj));
    }
    
    @Override
    protected void 
    encodeString(String obj, String name, Context ctx, int maxLength)
    throws EncodeException
    {
        putElement(name, obj);
    }
    
    @Override
    protected void 
    encodeOpaque(String data, String name, Context ctx, OpaqueSpec typeSpec)
    throws EncodeException
    {
        putElement(name, data);
    }

    @Override
    protected void 
    encodeArrayStart(String name, Context ctx, 
                     boolean isVariable, int actualLength)
    throws EncodeException
    {
    }
    
    @Override
    protected void 
    encodeArrayEnd(String name, Context ctx, 
                   boolean isVariable, int actualLength)
    throws EncodeException
    {
    }
    
    @Override
    protected void 
    encodeStructStart(String name, Context ctx, StructSpec spec)
    throws EncodeException
    {
        if (spec.isCollapsed)
            return;
        
        putOpenElement(name);
    }
    
    @Override
    protected void 
    encodeStructEnd(String name, Context ctx, StructSpec spec)
    throws EncodeException
    {
        if (spec.isCollapsed)
            return;
        
        putCloseElement(name);
    }
    
    @Override
    protected void 
    encodeUnionStart(String name, Context ctx, UnionSpec spec,
                     EnumValue discrimVal)
    throws EncodeException
    {
        putOpenElement(name);
        encodeEnum(discrimVal, spec.discrimName, Context.NONE);
    }
    
    @Override
    protected void 
    encodeUnionEnd(String name, Context ctx, UnionSpec spec)
    throws EncodeException
    {
        putCloseElement(name);
    }
    
    @Override
    protected void 
    encodeEnum(EnumValue enumVal, String name, Context ctx)
    throws EncodeException
    {
        emitString(String.format("<%1$s value='%2$d'>%3$s</%1$s>", 
                   name, enumVal.value, enumVal.name));
    }
    
    /**
     * This method is invoked to encode a pointer prefix.  Since XML
     * encodings do not use such prefixes, we do nothing here.  If the
     * pointer is non-null, we'll get a follow-on request to encode its
     * target object.  Otherwise, null pointers cause no encoding to
     * take place for XML.
     */
    @Override
    protected void 
    encodePointerPrefix(String name, Context ctx, boolean isNull)
    {
    }
}
