/******************************************************************************
 * Copyright 2015 NetApp, Inc. All rights reserved.                           *
 *                                                                            *
 * This file is confidential and a trade secret of NetApp, Inc.               *
 * The receipt of or possession of this file does not convey any rights to    *
 * reproduce or disclose its contents or to manufacture, use, or sell         *
 * anything it may describe, in whole, or in part, without the specific       *
 * written consent of NetApp, Inc.                                            *
 ******************************************************************************/

package devmgr.api.symbolutils;

import java.security.DigestException;
import java.security.Key;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Lightweight HMAC implementation using SHA256 for the hash function.
 * <P>
 * Where possible, this implementation matches the javax.crypto.Mac class implementation but to
 * reduce implementation size this class does not extend Mac. Ideally we would use the built in
 * "HmacSHA256" implementation, but the Solaris and AIX implementations do not allow 0 byte keys which are
 * required for our use in SYMbol.
 * <P>
 * REFERENCE: <BR>
 * http://csrc.nist.gov/publications/fips/fips198-1/FIPS-198-1_final.pdf
 */
public class HmacSha256 {
    private static final String HASH_ALGORITHM = "SHA-256";
    private static final int HASH_OUTPUT_SIZE = 32; // in bytes
    private static final int HASH_BLOCK_SIZE = 64; // in bytes
    private static final byte IPAD_BYTE = 0x36;
    private static final byte OPAD_BYTE = 0x5C;

    private MessageDigest sha256;

    /**
     * K0 in the FIPs document, generated from user supplied key
     */
    private byte[] key0;


    /**
     * Creates a new HmacSha256.
     *
     * @throws NoSuchAlgorithmException thrown if the digest algorithm is not available, this should
     *          not happen as SHA-256 is a required implementation.
     */
    public HmacSha256() throws NoSuchAlgorithmException {
        sha256 = MessageDigest.getInstance(HASH_ALGORITHM);
    }


    /**
     * Gets the size of the MAC output in bytes.
     *
     * @return size of the MAC output in bytes
     */
    public int getMacLength() { return HASH_OUTPUT_SIZE; }


    /**
     * Initializes this MAC object with the given key.
     * <P>
     * Must be called before any update or complete methods are called.
     *
     * @param key the Key instance
     */
    public void init(Key key) {
        key0 = key.getEncoded();

        if (key0.length > HASH_BLOCK_SIZE) {
            sha256.reset();
            key0 = sha256.digest(key.getEncoded());
        }

        if (key0.length < HASH_BLOCK_SIZE) {
            key0 = new byte[HASH_BLOCK_SIZE];
            System.arraycopy(key.getEncoded(), 0, key0, 0, key.getEncoded().length);
        }

        sha256.reset();
        sha256.update(xor(key0, IPAD_BYTE));
    }


    /**
     * Adds data upon which the MAC is to be calculated.
     * <P>
     * Adds to the "text" as defined in the fips document.<BR>
     * init method must be called before this method.
     *
     * @param input data to add to the MAC
     * @param offset 0 relative offset into the byte array from which to start
     * @param length amount of data from input to add
     * @throws IllegalStateException if this MAC has not been initialized
     */
    public void update(byte[] input, int offset, int length) {
        if (key0 == null) {
            throw new IllegalStateException("MAC not initialized");
        }

        sha256.update(input, offset, length);
    }


    /**
     * Finishes the HMAC calculations and generates the final HMAC digest.
     * <P>
     * setKey method must be called before this method.
     *
     * @param output byte array to put the generated MAC output
     * @param offset 0 relative offset into output to start putting the generated MAC output
     * @throws IllegalStateException if this MAC has not been initialized
     * @throws DigestException if an error occurs generating the MAC output
     */
    public void doFinal(byte[] output, int offset) throws DigestException {
        if (key0 == null) {
            throw new IllegalStateException("MAC not initialized");
        }

        byte[] innerDigest = sha256.digest();

        sha256.update(xor(key0, OPAD_BYTE));
        sha256.update(innerDigest);

        sha256.digest(output, offset, HASH_OUTPUT_SIZE);
    }


    /**
     * Performs an Exclusive-OR (XOR) on every index in the array with the specified byte value.
     *
     * @param array array to XOR
     * @param byteValue value to XOR against array bytes
     * @return XOR result array
     */
    private byte[] xor(byte[] array, byte byteValue) {
        byte[] result = new byte[array.length];
        for (int i = 0; i < result.length; i++) {
            result[i] = (byte) (array[i] ^ byteValue);
        }
        return result;
    }
}

