/**************************************************************************
 *                                                                         *
 *  Copyright 1998-2015 NetApp, Inc. All Rights Reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of NetApp, Inc.  The      *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of NetApp, Inc.                                        *
 *                                                                         *
 **************************************************************************/

package devmgr.api.symbolutils;

import java.net.Inet4Address;
import java.net.InetAddress;

import devmgr.api.rpc.RPCAuthGenerator;
import devmgr.api.rpc.RPCError;
import devmgr.api.rpc.protocol.RPCAuthType;
import devmgr.api.rpc.protocol.RPCOpaqueAuth;
import devmgr.api.runtime.ApiDefinition;
import devmgr.api.runtime.ApiUtil;
import devmgr.api.runtime.TypeSpec;
import devmgr.api.runtime.XdrEncoder;
import devmgr.api.runtime.XdrMessage;
import devmgr.api.symbol.IpAddressType;
import devmgr.api.symbol.IpVxAddress;
import devmgr.api.symbol.SAAttributeID;
import devmgr.api.symbol.SAData;
import devmgr.api.symbol.SAIdentifier;
import devmgr.api.symbol.SYMbolAuthentication;
import devmgr.api.symbol.SYMbolAuthenticationHMAC256;

/**
 * This class implements a generator object for the custom authentication mechanism for the SYMbol
 * protocol.  It supports both the legacy SHA-1 based MAC and the current HMAC-256 based MAC.
 * <P>
 * REFERENCE: <BR>
 * http://ictwiki.eng.netapp.com/index.php/SYMbol_security_enhancement
 */
public class SYMbolAuthGenerator implements RPCAuthGenerator {
    private final RPCAuthType authType;
    private final TypeSpec authTypeSpec;
    private final InetAddress clientAddress;
    private final SymbolMac mac;
    private final SAIdentifier arrayId;
    private long configGeneration;
    private byte[] password;
    
    private final String allZeroDigest;


    /**
     * Creates the SYMbol authentication generator object.  The authentication method chosen will be the strongest
     * supported for the array.
     *
     * @param client Client IP address.
     * @param arrayId Storage array identifier (world-wide name).
     * @exception RPCError If any unexpected error occurs.
     * @deprecated this constructor only support legacy SHA-1 authentication.  Use the constructor which takes an SAData
     *          instead as it will determine the highest authentication level supported for the array.
     */
    @Deprecated
    public SYMbolAuthGenerator(ApiDefinition apiDef, InetAddress client, SAIdentifier arrayId) throws RPCError {
        this.arrayId = arrayId;
        clientAddress = client;

        //Initialize the password and configuration generation number.
        password = new byte[0];
        configGeneration = 0;

        mac = new SymbolMacSha1();
        authType = RPCAuthType.AUTH_DIGEST;
        authTypeSpec = apiDef.getTypeSpec(SYMbolAuthentication.class);
        allZeroDigest = ApiUtil.opaqueToString(new byte[mac.getOutputSize()]);
    }


    /**
     * Creates the SYMbol authentication generator object.  The authentication method chosen will be the strongest
     * supported for the array.
     *
     * @param client Client IP address.
     * @param saData Storage array data containing the identifier (world-wide name) and attributes for determining
     *          authentication type
     * @exception RPCError If any unexpected error occurs.
     */
    public SYMbolAuthGenerator(ApiDefinition apiDef, InetAddress client, SAData saData) throws RPCError {
        arrayId = saData.getSaId();
        clientAddress = client;

        //Initialize the password and configuration generation number.
        password = new byte[0];
        configGeneration = 0;

        // ArrayAttributes is an attribute followed by a value so we skip 2 to ensure we aren't
        // checking any values.
        for (int i = 0; i < saData.getArrayAttributes().length; i += 2) {
            if (saData.getArrayAttributes()[i] == SAAttributeID.symbolAuthHmac256Support.value) {
                mac = new SymbolMacHmac256();
                authType = RPCAuthType.AUTH_DIGEST_HMAC256;
                authTypeSpec = apiDef.getTypeSpec(SYMbolAuthenticationHMAC256.class);
                allZeroDigest = ApiUtil.opaqueToString(new byte[mac.getOutputSize()]);
                return;
            }
        }

        // Didn't determine any other type, fall back to legacy mode
        mac = new SymbolMacSha1();
        authType = RPCAuthType.AUTH_DIGEST;
        authTypeSpec = apiDef.getTypeSpec(SYMbolAuthentication.class);
        allZeroDigest = ApiUtil.opaqueToString(new byte[mac.getOutputSize()]);
    }


    /**
     * Creates the "initial" authentication information for an RPC request message.  We build up a new opaque
     * authentication object using the most recent known information, and then return it to the user.
     *
     * @return The initial authentication data for the message.
     * @exception RPCError If any RPC error occurs.
     */
    public RPCOpaqueAuth getInitialAuthData() throws RPCError {
        return buildAuthObject(allZeroDigest);
    }


    /**
     * Generates the "final" authentication information for the specified RPC message.  We do this by calculating a
     * MAC over the entire message, and then constructing the authentication object the same way we did for the initial
     * phase, except that we use the computed digest instead of all-zeros.
     *
     * @param initial Initial authentication data used in the message.
     * @param msg Complete RPC message to be authenticated.
     * @return The final authentication data for the message.
     * @exception RPCError If any RPC error occurs.
     */
    public synchronized RPCOpaqueAuth getFinalAuthData(RPCOpaqueAuth initial, XdrMessage msg) throws RPCError {
        mac.setKey(password);
        mac.update(msg);
        byte[] digestBytes = mac.digest();
        return buildAuthObject(ApiUtil.opaqueToString(digestBytes));
    }


    /**
     * Sets the current value of the configuration generation number for use in generating authentication information.
     *
     * @param value New configuration generation number value to use.
     */
    public void setConfigGeneration(long value) { configGeneration = value; }


    /**
     * Sets the password value that is used to generate authentication information for outbound client messages.
     *
     * @param passwordString Password string for generating authentication data.
     */
    public synchronized void setPassword(String passwordString) {
        int len = 2 * passwordString.length();
        this.password = new byte[len];

        for (int i = 0; i < len / 2; i++) {
           int value = (int) passwordString.charAt(i);

           this.password[(2 * i) + 0] = (byte) (value >> 8);
           this.password[(2 * i) + 1] = (byte) (value >> 0);
        }
    }


    /**
     * Build the authentication object using the digest value provided by the caller.
     *
     * @param digestValue Digest value to plug into the object.
     * @return Constructed RPCOpaqueAuth object.
     * @throws RPCError if any errors occur while generating the auth data.
     */
    private RPCOpaqueAuth buildAuthObject(String digestValue) throws RPCError {
        Object authenticationObject;

        // Get the authentication object for the supported auth type
        switch (authType) {
            case AUTH_DIGEST:
                authenticationObject = buildSymbolAuthenticationSha1(digestValue);
                break;

            case AUTH_DIGEST_HMAC256:
                authenticationObject = buildSymbolAuthenticationHmac256(digestValue);
                break;

            default:
                assert false : authType;
                throw new RPCError(RPCError.ErrorType.AUTH_ALGORITHM);
        }

        // Convert the authentication object into an encoded XDR message as the encoded bytes are used later
        XdrMessage authenticationMessage;

        try {
            XdrEncoder encoder = new XdrEncoder();
            encoder.encode(authenticationObject, authTypeSpec);
            authenticationMessage = encoder.getMessage();
        }
        catch (Exception e) {
            RPCError wrapperException = new RPCError(RPCError.ErrorType.AUTH_ALGORITHM);
            wrapperException.initCause(e);
            throw wrapperException;
        }

        // Now extract it back out into normal in-memory format, i.e. as a Java String object.
        byte[] rawData = new byte[authenticationMessage.getMessageSize()];
        System.arraycopy(authenticationMessage.getContiguousBuffer(), 0, rawData, 0, rawData.length);
        String opaqueData = ApiUtil.opaqueToString(rawData);

        // Put the raw serialized data into the "info" field of the opaque authentication object.
        RPCOpaqueAuth opaqueAuth = new RPCOpaqueAuth();
        opaqueAuth.setType(authType);
        opaqueAuth.setInfo(opaqueData);

        return opaqueAuth;
    }


    /**
     * Builds the SYMbol authentication data structure for the legacy SHA-1 based MAC.
     *
     * @param digestValue digest value to plug into the object
     * @return SYMbol authentication data
     */
    private SYMbolAuthentication buildSymbolAuthenticationSha1(String digestValue) {
        // Turn the client IP address into an integer and save it.
        byte[] addressBytes = clientAddress.getAddress();
        int clientIp = 0;

        for (int i = 0; i < 4; i++) {
            clientIp = (clientIp << 8) | (addressBytes[i] & 0xFF);
        }

        SYMbolAuthentication symbolAuthentication = new SYMbolAuthentication();
        symbolAuthentication.setClientIp(clientIp);
        symbolAuthentication.setArrayId(arrayId);
        symbolAuthentication.setConfigGeneration(configGeneration);
        symbolAuthentication.setDigest(digestValue);

        return symbolAuthentication;
    }


    /**
     * Builds the SYMbol authentication data structure for the HMAC with SHA-256 based MAC.
     *
     * @param digestValue digest value to plug into the object
     * @return SYMbol authentication data
     */
    private SYMbolAuthenticationHMAC256 buildSymbolAuthenticationHmac256(String digestValue) {
        IpVxAddress clientIp = new IpVxAddress();

        if (clientAddress instanceof Inet4Address) {
            // IPv4 is treated special in the XDR encoding - it must be in dotted notation
            clientIp.setAddressType(IpAddressType.ipv4);
            clientIp.setIpv4Address(ByteUtils.getIpAddressString(clientAddress.getAddress()));
        }
        else {
            // IPv6 is not treated special in the XDR encoding - it is encoded like a standard opaque
            clientIp.setAddressType(IpAddressType.ipv6);
            clientIp.setIpv6Address(ApiUtil.opaqueToString(clientAddress.getAddress()));
        }

        SYMbolAuthenticationHMAC256 symbolAuthentication = new SYMbolAuthenticationHMAC256();
        symbolAuthentication.setClientIp(clientIp);
        symbolAuthentication.setArrayId(arrayId);
        symbolAuthentication.setConfigGeneration(configGeneration);
        symbolAuthentication.setDigest(digestValue);

        return symbolAuthentication;
    }
}
