/******************************************************************************
 * Copyright 2015 NetApp, Inc. All rights reserved.                           *
 *                                                                            *
 * This file is confidential and a trade secret of NetApp, Inc.               *
 * The receipt of or possession of this file does not convey any rights to    *
 * reproduce or disclose its contents or to manufacture, use, or sell         *
 * anything it may describe, in whole, or in part, without the specific       *
 * written consent of NetApp, Inc.                                            *
 ******************************************************************************/

package devmgr.api.symbolutils;

import devmgr.api.rpc.RPCError;

import javax.crypto.SecretKey;
import java.security.GeneralSecurityException;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

/**
 * SYMbol Message Authentication Code generator using the HMAC-256 for FIPS SP800-131A compliance.
 * <P>
 * REFERENCE: <BR>
 * http://ictwiki.eng.netapp.com/index.php/SYMbol_security_enhancement
 */
public class SymbolMacHmac256 extends SymbolMac {
    private static final String KEY_TYPE = "RAW";

    private HmacSha256 m_Hmac256;


    /**
     * Creates a new SymbolMacHmac256.
     *
     * @throws RPCError if an error occurs initializing the HMAC.
     */
    public SymbolMacHmac256() throws RPCError {
        try {
            m_Hmac256 = new HmacSha256();
        }
        catch (NoSuchAlgorithmException e) {
            // Shouldn't happen - HMAC with SHA-256 algorithm is required by all Java implementations
            RPCError error = new RPCError(RPCError.ErrorType.AUTH_ALGORITHM);
            error.initCause(e);
            throw error;
        }
    }


    @Override
    public int getOutputSize() {
        return m_Hmac256.getMacLength();
    }


    @Override
    public void setKey(byte[] key) throws RPCError {
        m_Hmac256.init(new HmacKey(key));
    }


    @Override
    public void update(byte[] input, int offset, int length) {
        m_Hmac256.update(input, offset, length);
    }


    @Override
    public void digest(byte[] output, int offset) throws RPCError {
        try {
            m_Hmac256.doFinal(output, offset);
        }
        catch (GeneralSecurityException e) {
            RPCError error = new RPCError(RPCError.ErrorType.AUTH_ALGORITHM);
            error.initCause(e);
            throw error;
        }
    }


    /**
     * Key implementation for the HMAC.
     * <P>
     * SecretKeySpec was preferred to be used, but it explicitly rejects empty keys which are valid
     * for HMAC.  There is a workaround for SecretKeySpec to use a dummy array, but that could be
     * made invalid with a JRE update as it takes advantage of an undocumented handling of the
     * extended constructor.
     * <P>
     * REFERENCE: <BR>
     * https://bugs.openjdk.java.net/browse/JDK-8020509
     */
    @SuppressWarnings("serial")
    private static class HmacKey implements SecretKey {
        private byte[] mi_Data;


        /**
         * Creates a new HmacKey.
         *
         * @param data raw key data
         */
        HmacKey(byte[] data) {
            assert data != null;
            mi_Data = new byte[data.length];
            System.arraycopy(data, 0, mi_Data, 0, data.length);
        }


        @Override
        public byte[] getEncoded() { return mi_Data; }


        @Override
        public String getAlgorithm() { return KEY_TYPE; }


        @Override
        public String getFormat() { return KEY_TYPE; }


        // In Java 8, Destroyable is added as a super Interface for SecretKey.  This method (without @Override
        // annotation) allows this to compile for either Java 7 or Java 8.
        public void destroy() {
            Arrays.fill(mi_Data, (byte) 0);
            mi_Data = null;
        }


        // In Java 8, Destroyable is added as a super Interface for SecretKey.  This method (without @Override
        // annotation) allows this to compile for either Java 7 or Java 8.
        public boolean isDestroyed() {
            return mi_Data == null;
        }
    }
}
